'use strict';

/*global
    jQuery, wp, grimlock_widgets
 */

/**
 * widgets.js
 *
 * Widgets enhancements for a better user experience.
 */

(function($){
    $(document).ready(function() {

        /*
         * GalleryDetailsMediaFrame class
         *
         * The class to create the media gallery modal
         */
        var GalleryDetailsMediaFrame = wp.media.view.MediaFrame.Post.extend( {

            /**
             * Create the default states.
             *
             * @since 4.9.0
             * @returns {void}
             */
            createStates: function createStates() {
                this.states.add([
                    new wp.media.controller.Library({
                        id:         'gallery',
                        title:      wp.media.view.l10n.createGalleryTitle,
                        priority:   40,
                        toolbar:    'main-gallery',
                        filterable: 'uploaded',
                        multiple:   'add',
                        editable:   true,

                        library:  wp.media.query(_.defaults({
                            type: 'image'
                        }, this.options.library ) )
                    }),

                    // Gallery states.
                    new wp.media.controller.GalleryEdit({
                        library: this.options.selection,
                        editing: this.options.editing,
                        menu:    'gallery'
                    }),

                    new wp.media.controller.GalleryAdd()
                ]);
            }
        } );

        /*
         * Gallery class
         *
         * The class that represents a gallery
         */
        var Gallery = function ($wrapper, props, selectedAttachments) {
            this.$input = $wrapper.find('.gallery-props');
            this.$previewContainer = $wrapper.find('.media-widget-preview');
            this.props = props;
            this.mediaFrame = false;
            this.selectedAttachments = selectedAttachments;

            this.renderPreview = function() {
                if (this.$previewContainer.length && this.selectedAttachments.length) {

                    var previewImages = '';

                    this.selectedAttachments.each(function(attachment, index) {

                        var attachmentData = attachment.toJSON();

                        if (index < 6) {
                            var img = '';

                            if (attachmentData.sizes.thumbnail) {
                                img = '<img src="' + attachmentData.sizes.thumbnail.url + '" width="' + attachmentData.sizes.thumbnail.width + '" height="' + attachmentData.sizes.thumbnail.height + '" alt="" />';
                            }
                            else {
                                img = '<img src="' + attachmentData.url + '" alt="" />';
                            }

                            if (index === 5 && this.selectedAttachments.length > 6) {
                                img += '<div class="gallery-icon-placeholder"><p class="gallery-icon-placeholder-text">+' +  (this.selectedAttachments.length - 5) + '</p></div>';
                            }

                            previewImages += '<dl class="gallery-item"><dt class="gallery-icon">' + img + '</dt></dl>';
                        }
                    }.bind(this));

                    this.$previewContainer.html('<div class="gallery media-widget-gallery-preview">' + previewImages + '</div>');
                }
            }.bind(this);

            // Callback function when updating the gallery
            this.onUpdate = function(newSelection) {
                var state = this.mediaFrame.state(), resultSelection;

                resultSelection = newSelection || state.get('selection');
                if (!resultSelection) {
                    return;
                }

                // Save gallery props
                if (resultSelection.gallery) {
                    this.props = _.extend(this.props, resultSelection.gallery.toJSON());
                }

                // Update selected attachments
                this.selectedAttachments.reset(resultSelection.models);

                // Update widget instance.
                this.props.ids = _.pluck(resultSelection.models, 'id');

                this.$input.val(JSON.stringify(this.props)).change();
                this.renderPreview();
            }.bind(this);

            // Function to open the media gallery modal
            this.openMediaFrame = function() {

                var selection = new wp.media.model.Selection(selectedAttachments.models, {
                    multiple: true
                });
                selection.gallery = new Backbone.Model(this.props);

                if (!this.props.ids) {
                    this.mediaFrame = new GalleryDetailsMediaFrame({
                        frame: 'select',
                        selection: selection,
                        mimeType: 'image',
                        metadata: this.props,
                        state: 'gallery'
                    });

                }
                else {
                    this.mediaFrame = new GalleryDetailsMediaFrame({
                        frame: 'manage',
                        selection: selection,
                        mimeType: 'image',
                        metadata: this.props,
                        editing:   true,
                        multiple:  true,
                        state: 'gallery-edit'
                    });
                }

                wp.media.frame = this.mediaFrame;
                this.mediaFrame.on('update', this.onUpdate);
                this.mediaFrame.$el.addClass('media-widget');
                this.mediaFrame.open();
            }.bind(this);

            this.renderPreview();
        };

        var defaultGalleryProps = {
            columns: 3,
            ids: '',
            link: "post",
            size: "thumbnail",
            type: "default",
            widget_id: "media_gallery-2",
            _orderbyRandom: false
        };

        var galleries = {};

        var initGallery = function ($widgetContainer) {
            var $galleryFieldWrapper = $widgetContainer.find('.grimlock_gallery_section_widget-gallery');

            // Check if the opened widget is a Grimlock Gallery Section widget
            if ($galleryFieldWrapper.length) {

                // Get the input containing the gallery props
                var $input = $galleryFieldWrapper.find('.gallery-props');
                var galleryProps = _.extend({}, defaultGalleryProps, JSON.parse($input.val()));

                // Get the widget id
                var widgetId = $widgetContainer.attr('id');

                var selectedAttachments = new wp.media.model.Attachments();

                // If the gallery is not empty, preload the attachments
                if (galleryProps.ids) {
                    var selectedAttachmentsQuery = wp.media.query({
                        order: 'ASC',
                        orderby: 'post__in',
                        perPage: -1,
                        post__in: galleryProps.ids,
                        query: true,
                        type: 'image'
                    });
                    selectedAttachmentsQuery.more().done(function () {
                        selectedAttachments.reset(selectedAttachmentsQuery.models);
                        galleries[widgetId] = new Gallery($galleryFieldWrapper, galleryProps, selectedAttachments);
                        $galleryFieldWrapper.find('.select-media').removeClass('hidden');
                    });
                }
                else {
                    galleries[widgetId] = new Gallery($galleryFieldWrapper, galleryProps, selectedAttachments);
                }
            }
        };

        // Preload gallery attachments when opening a gallery section widget
        var widgetContainers = $('.widgets-holder-wrap:not(#available-widgets)').find( 'div.widget' );

        widgetContainers.one('click.toggle-widget-expanded', function() {
            initGallery($(this));
        });

        $(document).on('widget-added', function (event, $widgetContainer) {
            initGallery($widgetContainer);
        });

        $(document).on('click', '.grimlock_gallery_section_widget-gallery .select-media, .grimlock_gallery_section_widget-gallery .media-widget-preview:not( .spinner)', function(e) {
            e.preventDefault();
            var $widgetContainer = $(this).closest('div.widget');
            if ($widgetContainer.length) {
                var widgetId = $widgetContainer.attr('id');
                if (galleries[widgetId]) {
                    galleries[widgetId].openMediaFrame();
                }
            }
        });

        $(document).on('widget-updated', function (event, $widgetContainer) {
            var $galleryFieldWrapper = $widgetContainer.find('.grimlock_gallery_section_widget-gallery');

            // Check if the updated widget is a Grimlock Gallery Section widget
            if ($galleryFieldWrapper.length) {

                // Get the widget id
                var widgetId = $widgetContainer.attr('id');

                if (galleries[widgetId]) {
                    // Re-render the previews
                    galleries[widgetId] = new Gallery($galleryFieldWrapper, galleries[widgetId].props, galleries[widgetId].selectedAttachments);
                }
            }
        });
    });
})(jQuery);
