<?php
// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Grimlock_Gallery_Section_Block
 *
 * @author  themosaurus
 * @since   1.0.6
 * @package grimlock-gallery/inc
 */
class Grimlock_Gallery_Section_Block extends Grimlock_Section_Block {

	/**
	 * Setup class.
	 *
	 * @param string $type Block type
	 * @param string $domain Block domain
	 *
	 * @since 1.1.5
	 */
	public function __construct( $type = 'gallery-section', $domain = 'grimlock-gallery' ) {
		parent::__construct( $type, $domain );

		// General Panel
		remove_filter( "{$this->id_base}_general_panel_fields",  array( $this, 'add_thumbnail_field'          ), 100 );
		remove_filter( "{$this->id_base}_general_panel_fields",  array( $this, 'add_thumbnail_size_field'     ), 100 );
		add_filter( "{$this->id_base}_general_panel_fields",     array( $this, 'add_gallery_images_field'     ), 100 );
		add_filter( "{$this->id_base}_general_panel_fields",     array( $this, 'add_gallery_columns_field'    ), 101 );
		add_filter( "{$this->id_base}_general_panel_fields",     array( $this, 'add_gallery_link_to_field'    ), 102 );
		add_filter( "{$this->id_base}_general_panel_fields",     array( $this, 'add_gallery_image_size_field' ), 103 );
		if ( class_exists( 'Jetpack' ) ) {
			add_filter( "{$this->id_base}_general_panel_fields", array( $this, 'add_gallery_type_field'       ), 104 );
		}
	}

	/**
	 * Add an image field to set the gallery images of the section
	 *
	 * @param array $fields Array of block fields
	 *
	 * @return array Modified array of block fields
	 */
	public function add_gallery_images_field( $fields ) {
		$fields[] = $this->image_field( apply_filters( "{$this->id_base}_gallery_images_field_args", array(
			'name'         => 'gallery_images',
			'label'        => esc_html__( 'Gallery', 'grimlock-gallery' ),
			'gallery'      => true,
			'return_as_id' => true,
		) ) );

		return $fields;
	}

	/**
	 * Add a range field to set the number of columns for the gallery
	 *
	 * @param array $fields Array of block fields
	 *
	 * @return array Modified array of block fields
	 */
	public function add_gallery_columns_field( $fields ) {
		$fields[] = $this->range_field( apply_filters( "{$this->id_base}_gallery_columns_field_args", array(
			'name'  => 'gallery_columns',
			'min'   => 1,
			'max'   => 4,
			'step'  => 1,
			'label' => esc_html__( 'Gallery Columns', 'grimlock-gallery' ),
			'conditional_logic' => array(
				array(
					'field'    => 'gallery_images',
					'operator' => '!=',
					'value'    => false,
				),
			),
		) ) );

		return $fields;
	}

	/**
	 * Add a select field to set what the gallery images should link to
	 *
	 * @param array $fields Array of block fields
	 *
	 * @return array Modified array of block fields
	 */
	public function add_gallery_link_to_field( $fields ) {
		// Get gallery types
		$fields[] = $this->select_field( apply_filters( "{$this->id_base}_gallery_link_to_field_args", array(
			'name'  => 'gallery_link_to',
			'label' => esc_html__( 'Gallery Images Link to', 'grimlock-gallery' ),
			'choices' => array(
				'post' => esc_html__( 'Attachment Page', 'grimlock-gallery' ),
				'file' => esc_html__( 'Media File', 'grimlock-gallery' ),
				'none' => esc_html__( 'None', 'grimlock-gallery' ),
			),
			'conditional_logic' => array(
				array(
					'field'    => 'gallery_images',
					'operator' => '!=',
					'value'    => false,
				),
			),
		) ) );

		return $fields;
	}

	/**
	 * Add a select field to set the image size to use for the gallery images
	 *
	 * @param array $fields Array of block fields
	 *
	 * @return array Modified array of block fields
	 */
	public function add_gallery_image_size_field( $fields ) {
		// Get named image sizes
		$choices = apply_filters(
			'image_size_names_choose',
			array(
				''          => esc_html__( 'Default', 'grimlock' ),
				'full'      => __( 'Full Size' ),
				'thumbnail' => __( 'Thumbnail' ),
				'medium'    => __( 'Medium' ),
				'large'     => __( 'Large' ),
			)
		);

		// Add non-named image sizes
		$image_sizes = get_intermediate_image_sizes();
		foreach ( $image_sizes as $image_size ) {
			if ( isset( $choices[ $image_size ] ) ) {
				continue;
			}

			$label = str_replace( '-', ' ', $image_size );
			$label = ucwords( $label );
			$choices[ $image_size ] = $label;
		}

		$fields[] = $this->select_field( apply_filters( "{$this->id_base}_gallery_image_size_field_args", array(
			'name'  => 'gallery_image_size',
			'label' => esc_html__( 'Gallery images size', 'grimlock-gallery' ),
			'choices' => $choices,
			'conditional_logic' => array(
				array(
					'field'    => 'gallery_images',
					'operator' => '!=',
					'value'    => false,
				),
			),
		) ) );

		return $fields;
	}

	/**
	 * Add a select field to set the type of gallery to use (only if Jetpack is activated)
	 *
	 * @param array $fields Array of block fields
	 *
	 * @return array Modified array of block fields
	 */
	public function add_gallery_type_field( $fields ) {
		// Get gallery types
		$choices = apply_filters( 'jetpack_gallery_types', array( 'default' => __( 'Thumbnail Grid', 'jetpack' ) ) );

		$fields[] = $this->select_field( apply_filters( "{$this->id_base}_gallery_type_field_args", array(
			'name'  => 'gallery_type',
			'label' => esc_html__( 'Gallery type', 'grimlock-gallery' ),
			'choices' => $choices,
			'conditional_logic' => array(
				array(
					'field'    => 'gallery_images',
					'operator' => '!=',
					'value'    => false,
				),
			),
		) ) );

		return $fields;
	}

	/**
	 * Get default field values for the block
	 *
	 * @return array Array of default field values
	 */
	public function get_defaults() {
		$defaults = parent::get_defaults();

		return array_merge( $defaults, array(
			'gallery_images'     => array(),
			'gallery_columns'    => 3,
			'gallery_link_to'    => 'none',
			'gallery_image_size' => 'medium',
			'gallery_type'       => 'default',
		) );
	}

	/**
	 * Render the Gutenberg block
	 *
	 * @param $attributes
	 * @param $content
	 *
	 * @return string
	 */
	public function render_block( $attributes, $content ) {
		$attributes = $this->sanitize_attributes( $attributes );
		ob_start();
		do_action( 'grimlock_gallery_section', apply_filters( "{$this->id_base}_component_args", $this->get_component_args( $attributes ), $attributes ) );
		return ob_get_clean();
	}

	/**
	 * Get the component args
	 *
	 * @param array $attributes Block attributes
	 *
	 * @return array Component args
	 */
	protected function get_component_args( $attributes ) {
		$args = parent::get_component_args( $attributes );

		return array_merge( $args, array(
			'gallery' => ! empty( $attributes['gallery_images'] ) ? array(
				'columns'        => $attributes['gallery_columns'],
				'ids'            => implode( ',', $attributes['gallery_images'] ),
				'link'           => $attributes['gallery_link_to'],
				'size'           => $attributes['gallery_image_size'],
				'type'           => $attributes['gallery_type'],
				'_orderbyRandom' => false,
			) : false,
		) );
	}

	/**
	 * Handles sanitizing attributes for the current block instance.
	 *
	 * @param array $new_attributes New attributes for the current block instance
	 *
	 * @return array Attributes to save
	 */
	public function sanitize_attributes( $new_attributes ) {
		$attributes = parent::sanitize_attributes( $new_attributes );

		$attributes['gallery_images']     = isset( $new_attributes['gallery_images'] ) ? $new_attributes['gallery_images'] : array();
		$attributes['gallery_columns']    = isset( $new_attributes['gallery_columns'] ) ? intval( $new_attributes['gallery_columns'] ) : 0;
		$attributes['gallery_link_to']    = isset( $new_attributes['gallery_link_to'] ) ? sanitize_text_field( $new_attributes['gallery_link_to'] ) : '';
		$attributes['gallery_image_size'] = isset( $new_attributes['gallery_image_size'] ) ? sanitize_text_field( $new_attributes['gallery_image_size'] ) : '';
		$attributes['gallery_type']       = isset( $new_attributes['gallery_type'] ) ? sanitize_text_field( $new_attributes['gallery_type'] ) : '';

		return $attributes;
	}
}

return new Grimlock_Gallery_Section_Block();
